import numpy.testing as npt

from normalisation.normalisation import *
from normalisation.data_model import compute_sum_by_fragments, merge_mq_metadata

raw_df = pd.DataFrame({'Original Filename': [1, 1, 2, 2, 3, 4, 1, 2, 3, 4],
                       'Sample Name': ['a_std', 'a_std', 'b_std', 'b_std', 'a',
                                       'b', 'a_std', 'b_std',
                                       'a', 'b'],
                       'Area': [5, 8, 10, 12, 15, 20, 25, 30, 0, 40],
                       'Component Name': ['P', 'P1', 'P1', 'P2', 'P', 'P1',
                                          'Q', 'Q1', 'Q2', 'Q1']})

metadata_mq = pd.DataFrame(
    {'Component Name': ['P', 'P1', 'P2', 'Q', 'Q1', 'Q2'],
     'Unlabeled Fragment': ['P', 'P', 'P', 'Q', 'Q', 'Q']})

metadata_std = pd.DataFrame({'Original Filename': [1, 1, 2, 2, 3, 4, 1, 2, 3,
                                                   4],
                             'Sample Name': ['a_std', 'a_std', 'b_std', 'b_std',
                                             'a', 'b', 'a_std', 'b_std', 'a',
                                             'b'],
                             'Unlabeled Fragment': ['P', 'P', 'P', 'P', 'P',
                                                    'P',
                                                    'Q', 'Q', 'Q', 'Q'],
                             'Concentration': [1, 1, 2, 2, 1, 2, 1, 2, 1, 2],
                             'Unit': ['M', 'M', 'M', 'M', 'M', 'M', 'M', 'M',
                                      'M', 'M']})


def test_calculate_normalising_factor():
    df = merge_mq_metadata(raw_df, metadata_mq)
    df = compute_sum_by_fragments(df)
    norm_factor, outlier_samples, max_intensity_acc_sample = \
        calculate_normalising_factor(df, 'Q', None)
    assert all([norm_factor == [0.625, 0.75, 0.0, 1.0],
                outlier_samples.columns.tolist() == ['Original Filename',
                                                     'Sample '
                                                     'Name',
                                                     'Unlabeled Fragment',
                                                     'Area',
                                                     'Component Name',
                                                     'Concentration'],
                max_intensity_acc_sample.columns.tolist() == [
                    'Original Filename', 'Sample '
                                         'Name',
                    'Unlabeled Fragment', 'Area',
                    'Component Name',
                    'Concentration']])


def test_normalised_data():
    df = merge_mq_metadata(raw_df, metadata_mq)
    df = compute_sum_by_fragments(df)
    norm_factor, outliers_data, accepted_data = calculate_normalising_factor(
        df, 'Q', None)
    norm_intensity = normalised_data(df, 'P', norm_factor)
    assert norm_intensity.tolist() == [20.800000000000001, 29.333333333333332,
                                       0.0,
                                       20.0]


def test_normalized_intensity_df():
    df = normalized_intensity_df(raw_df, metadata_mq, 'Q', None)
    expected_df = pd.DataFrame({'Original Filename': [1, 2, 3, 4, 1, 2, 3,
                                                      4],
                                'Sample Name': ['a_std', 'b_std', 'a', 'b',
                                                'a_std',
                                                'b_std', 'a', 'b'],
                                'Unlabeled Fragment': ['P', 'P', 'P', 'P',
                                                       'Q', 'Q', 'Q', 'Q'],
                                'Raw_Area': [13, 22, 15, 20, 25, 30, 0, 40],
                                'Component Name': ['P', 'P1',
                                                   'P', 'P1', 'Q', 'Q1', 'Q2',
                                                   'Q1'],
                                'Concentration': [-1, -1, -1, -1, -1, -1, -1,
                                                  -1],
                                'Area': [20.8, 29.333333333333332, 0.0,
                                         20.0, 40.0, 40.0, 0.0, 40.0]})
    expected_df = expected_df[['Original Filename', 'Sample Name',
                               'Unlabeled Fragment', 'Raw_Area',
                               'Component Name',
                               'Concentration', 'Area']]
    npt.assert_array_equal(df, expected_df)


def test_get_normalized_biosample_data():
    standard_df, biosamples_df = get_normalized_biosample_data(raw_df,
                                                               metadata_std,
                                                               metadata_mq, 'Q',
                                                               None)
    expected_standard_df = pd.DataFrame({'Original Filename': [1, 2, 1, 2],
                                         'Sample Name': ['a_std', 'b_std',
                                                         'a_std', 'b_std'],
                                         'Unlabeled Fragment': ['P', 'P', 'Q',
                                                                'Q'],
                                         'Raw_Area': [13, 22, 25, 30],
                                         'Component Name': ['P', 'P1', 'Q',
                                                            'Q1'],
                                         'Concentration': [1, 2, 1, 2],
                                         'Area': [20.800000, 29.333333,
                                                  40.00000,
                                                  40.00000],
                                         'Unit': ['M', 'M', 'M', 'M']})
    expected_standard_df = expected_standard_df[['Original Filename', 'Sample '
                                                                      'Name',
                                                 'Unlabeled Fragment',
                                                 'Raw_Area',
                                                 'Component Name', 'Area',
                                                 'Concentration', 'Unit']]

    expected_biosamples_df = pd.DataFrame({'Original Filename': [3, 4, 3, 4],
                                           'Sample Name': ['a', 'b', 'a', 'b'],
                                           'Unlabeled Fragment': ['P', 'P', 'Q',
                                                                  'Q'],
                                           'Raw_Area': [15, 20, 0, 40],
                                           'Component Name': ['P', 'P1', 'Q2',
                                                              'Q1'],
                                           'Concentration': [-1, -1, -1, -1],
                                           'Area': [0.0, 20.000, 0.0, 40.000]})
    expected_biosamples_df = expected_biosamples_df[
        ['Original Filename', 'Sample '
                              'Name',
         'Unlabeled Fragment',
         'Raw_Area',
         'Component Name',
         'Concentration', 'Area']]
    assert all([np.array_equal(standard_df.size, expected_standard_df.size),
                np.array_equal(biosamples_df, expected_biosamples_df)])


def test_get_outlier_samples():
    df = merge_mq_metadata(raw_df, metadata_mq)
    df = compute_sum_by_fragments(df)
    internal_std_df = df[df[constants.PARENT] == 'Q']
    outlier_samples = get_outlier_samples(internal_std_df)
    assert outlier_samples.columns.tolist() == ['Original Filename', 'Sample '
                                                                     'Name',
                                                'Unlabeled Fragment', 'Area',
                                                'Component Name',
                                                'Concentration']


def test_get_max_intensity_accepted_sample():
    df = merge_mq_metadata(raw_df, metadata_mq)
    df = compute_sum_by_fragments(df)
    accepted_sample = get_max_intensity_accepted_sample(df)
    assert accepted_sample.columns.tolist() == ['Original Filename', 'Sample '
                                                                     'Name',
                                                'Unlabeled Fragment', 'Area',
                                                'Component Name',
                                                'Concentration']


def test_get_outliers_data():
    outliers_data = get_outliers_data(raw_df, metadata_mq)
    assert outliers_data == [
        {'default_sample': [{'Original Filename': 2, 'Concentration': -1}],
         'outlier_samples': [], 'name': 'P'},
        {'default_sample': [{'Original Filename': 4, 'Concentration': -1}],
         'outlier_samples': [], 'name': 'Q'}]